// OpenSSLWrapper.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "OpenSSLWrapper.h"

#include "IkAES.h"
#include "IkRSA.h"
#include "NpCrypt.h"

#if defined(WIN32)

#if defined(_DEBUG)
#pragma comment(lib, "IkOpenSSLWrapperD.lib")
#else
#pragma comment(lib, "IkOpenSSLWrapper.lib")
#endif


#define Log(x) OutputDebugString(x ## "\n")

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

#else
#define Log(x)
#endif // WIN32

OpenSSL_AES::OpenSSL_AES()
{
	Log("OpenSSL_AES::OpenSSL_AES");
	
	m_delegator = reinterpret_cast<void*>(new CIkAES);
}

OpenSSL_AES::~OpenSSL_AES()
{
	Log("OpenSSL_AES::~OpenSSL_AES");

	delete (CIkAES*)m_delegator;
	m_delegator = NULL;
}

int OpenSSL_AES::GenerateNewCryptKey()
{
	Log("OpenSSL_AES::GenerateNewCryptKey");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->GenerateNewCryptKey() : -1;
}

const BYTE * OpenSSL_AES::GetCryptKey()
{
	Log("OpenSSL_AES::GetCryptKey");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->GetCryptKey() : NULL;
}

bool OpenSSL_AES::SetCryptKey(const BYTE * pKey)
{
	Log("OpenSSL_AES::SetCryptKey");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->SetCryptKey(pKey) : false;
}

const BYTE * OpenSSL_AES::GetInitialVector()
{
	Log("OpenSSL_AES::GetInitialVector");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->GetInitialVector() : NULL;
}

bool OpenSSL_AES::SetInitialVector(const BYTE * pKey)
{
	Log("OpenSSL_AES::SetInitialVector");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->SetInitialVector(pKey) : false;
}

int OpenSSL_AES::Encrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer)
{
	Log("OpenSSL_AES::Encrypt");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->Encrypt(pSrc, nSrcLen, pOutBuffer) : -1;
}

int OpenSSL_AES::Decrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer)
{
	Log("OpenSSL_AES::Decrypt");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->Decrypt(pSrc, nSrcLen, pOutBuffer) : -1;
}

int OpenSSL_AES::EncryptedSizeGuess(int nSourceDataSize)
{
	Log("OpenSSL_AES::EncryptedSizeGuess");
	CIkAES* ikAES = reinterpret_cast<CIkAES*>(m_delegator);
	return ikAES ? ikAES->EncryptedSizeGuess(nSourceDataSize) : -1;
}


OpenSSL_RSA::OpenSSL_RSA()
{
	Log("OpenSSL_RSA::OpenSSL_RSA");
	m_delegator = reinterpret_cast<void*>(new CIkRSA);
}

OpenSSL_RSA::~OpenSSL_RSA()
{
	Log("OpenSSL_RSA::~OpenSSL_RSA");
	delete (CIkRSA*)m_delegator;
	m_delegator = NULL;
}

int OpenSSL_RSA::sRSA_EncryptByPublicKey(const BYTE * pRSA_PublicKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::sRSA_EncryptByPublicKey");
	return CIkRSA::sRSA_EncryptByPublicKey(pRSA_PublicKey, nKeyLen, pSrcData, nSrcLen, pOutBuffer, nOutBufferLen);
}

int OpenSSL_RSA::sRSA_DecryptByPublicKey(const BYTE * pRSA_PublicKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::sRSA_DecryptByPublicKey");
	return CIkRSA::sRSA_DecryptByPublicKey(pRSA_PublicKey, nKeyLen, pSrcData, nSrcLen, pOutBuffer, nOutBufferLen);
}

int OpenSSL_RSA::sRSA_EncryptByPrivateKey(const BYTE * pRSA_PrivateKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::sRSA_EncryptByPrivateKey");
	return CIkRSA::sRSA_EncryptByPrivateKey(pRSA_PrivateKey, nKeyLen, pSrcData, nSrcLen, pOutBuffer, nOutBufferLen);
}

int OpenSSL_RSA::sRSA_DecryptByPrivateKey(const BYTE * pRSA_PrivateKey, int nKeyLen, const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::sRSA_DecryptByPrivateKey");
	return CIkRSA::sRSA_DecryptByPrivateKey(pRSA_PrivateKey, nKeyLen, pSrcData, nSrcLen, pOutBuffer, nOutBufferLen);
}

bool OpenSSL_RSA::Create_InternalKey(int seed)
{
	Log("OpenSSL_RSA::Create_InternalKey");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->Create_InternalKey(seed) : false;
}

bool OpenSSL_RSA::Initialize_By_ExternalKey(const BYTE * pRSA_PublicKey, int nPubKeyLen, const BYTE * pRSA_PrivateKey, int nPriKeyLen)
{
	Log("OpenSSL_RSA::Initialize_By_ExternalKey");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->Initialize_By_ExternalKey(pRSA_PublicKey, nPubKeyLen, pRSA_PrivateKey, nPriKeyLen) : false;
}

int OpenSSL_RSA::GetPublicKey(BYTE ** ppOutBuffer, void* (*mem_allocator)(size_t))
{
	Log("OpenSSL_RSA::GetPublicKey");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->GetPublicKey(ppOutBuffer, mem_allocator) : -1;
}

int OpenSSL_RSA::GetPrivateKey(BYTE ** ppOutBuffer, void* (*mem_allocator)(size_t))
{
	Log("OpenSSL_RSA::GetPrivateKey");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->GetPrivateKey(ppOutBuffer, mem_allocator) : -1;
}
	
int OpenSSL_RSA::Public_Encrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::Public_Encrypt");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->Public_Encrypt(pSrcData, nSrcLen, pOutBuffer, nOutBufferLen) : -1;
}

int OpenSSL_RSA::Public_Decrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::Public_Decrypt");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->Public_Decrypt(pSrcData, nSrcLen, pOutBuffer, nOutBufferLen) : -1;
}

int OpenSSL_RSA::Private_Encrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::Private_Encrypt");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->Private_Encrypt(pSrcData, nSrcLen, pOutBuffer, nOutBufferLen) : -1;
}

int OpenSSL_RSA::Private_Decrypt(const BYTE * pSrcData, int nSrcLen, BYTE * pOutBuffer, int nOutBufferLen)
{
	Log("OpenSSL_RSA::Private_Decrypt");
	CIkRSA* ikRSA = reinterpret_cast<CIkRSA*>(m_delegator);
	return ikRSA ? ikRSA->Private_Decrypt(pSrcData, nSrcLen, pOutBuffer, nOutBufferLen) : -1;
}

OpenSSL_Crypt::OpenSSL_Crypt()
{
	Log("OpenSSL_Crypt::OpenSSL_Crypt");
	m_delegator = reinterpret_cast<void*>(new CNpCrypt);
}

OpenSSL_Crypt::~OpenSSL_Crypt()
{
	Log("OpenSSL_Crypt::~OpenSSL_Crypt");
	delete (CNpCrypt*)m_delegator;
	m_delegator = NULL;
}

void OpenSSL_Crypt::Init(int nEncType, const BYTE * pAesCryptKey)
{
	Log("OpenSSL_Crypt::Init");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	if (npCrypt) npCrypt->Init(nEncType, pAesCryptKey) ;
}

int OpenSSL_Crypt::GetEncType()
{
	Log("OpenSSL_Crypt::GetEncType");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	return npCrypt ? npCrypt->GetEncType() : -1;
}

int OpenSSL_Crypt::GetAesKeySize()
{
	Log("OpenSSL_Crypt::GetAesKeySize");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	return npCrypt ? CIkAES::IKAES_CRYPT_KEY_SIZE : -1;
}

const BYTE * OpenSSL_Crypt::GetAesKey()
{
	Log("OpenSSL_Crypt::GetAesKey");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	return npCrypt ? npCrypt->GetIkAES().GetCryptKey() : NULL;
}

int OpenSSL_Crypt::AesEncrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer)
{
	Log("OpenSSL_Crypt::AesEncrypt 1");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	return npCrypt ? npCrypt->AesEncrypt(pSrc, nSrcLen, pOutBuffer) : -1;
}

int OpenSSL_Crypt::AesDecrypt(const BYTE * pSrc, int nSrcLen, BYTE * pOutBuffer)
{
	Log("OpenSSL_Crypt::AesDecrypt 2");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	return npCrypt ? npCrypt->AesDecrypt(pSrc, nSrcLen, pOutBuffer) : -1;
}

int OpenSSL_Crypt::AesEncryptSizeGuess(int nSrcLen)
{
	Log("OpenSSL_Crypt::AesEncryptSizeGuess");
	CNpCrypt* npCrypt = reinterpret_cast<CNpCrypt*>(m_delegator);
	return npCrypt ? npCrypt->AesEncryptSizeGuess(nSrcLen) : -1;
}
